@@ -1,8 +1,8 @@
 // API references:
-// 
-// * http://dev.w3.org/html5/websockets/
-// * http://dvcs.w3.org/hg/domcore/raw-file/tip/Overview.html#interface-eventtarget
-// * http://dvcs.w3.org/hg/domcore/raw-file/tip/Overview.html#interface-event
+//
+// * https://html.spec.whatwg.org/multipage/comms.html#network
+// * https://dom.spec.whatwg.org/#interface-eventtarget
+// * https://dom.spec.whatwg.org/#interface-event
 
 var util   = require('util'),
     driver = require('websocket-driver'),
@@ -16,15 +16,16 @@ var WebSocket = function(request, socket, body, protocols, options) {
 
   var self = this;
   if (!this._stream || !this._stream.writable) return;
+  if (!this._stream.readable) return this._stream.end();
 
   var catchup = function() { self._stream.removeListener('data', catchup) };
   this._stream.on('data', catchup);
 
-  this._driver.io.write(body);
   API.call(this, options);
 
   process.nextTick(function() {
     self._driver.start();
+    self._driver.io.write(body);
   });
 };
 util.inherits(WebSocket, API);
@@ -33,9 +34,12 @@ WebSocket.isWebSocket = function(request) {
   return driver.isWebSocket(request);
 };
 
+WebSocket.validateOptions = function(options, validKeys) {
+  driver.validateOptions(options, validKeys);
+};
+
 WebSocket.WebSocket   = WebSocket;
 WebSocket.Client      = require('./websocket/client');
 WebSocket.EventSource = require('./eventsource');
 
 module.exports        = WebSocket;
-
